import { useCallback, useEffect, useState, ReactNode } from "react";
import { userContext } from "@ess-ics/ce-ui-common";
import {
  useLoginMutation,
  useLogoutMutation,
  useInfoFromUserNameQuery,
  useGetUserRolesQuery,
  UserInfoResponse,
  GetUserRolesApiResponse
} from "../store/deployApi";
import { getErrorState } from "../components/common/Alerts/AlertsData";

export function UserProvider({ children }: { children: ReactNode }) {
  const [user, setUser] = useState<UserInfoResponse | undefined>(undefined);
  const [userRoles, setUserRoles] = useState<
    GetUserRolesApiResponse | undefined
  >(undefined);

  const [
    callLogin,
    { error: loginError, isLoading: loginLoading, reset: resetLogin }
  ] = useLoginMutation();
  const [callLogout, { isLoading: logoutLoading }] = useLogoutMutation();
  const {
    data: userResponse,
    isLoading: userLoading,
    refetch: refetchUserInfo
  } = useInfoFromUserNameQuery({});
  const {
    data: userRolesResponse,
    isLoading: userRolesLoading,
    refetch: refetchUserRoles
  } = useGetUserRolesQuery(undefined);

  const initialized = Boolean(
    !loginLoading || !logoutLoading || !userLoading || !userRolesLoading
  );

  const login = useCallback(
    async (username: string, password: string) => {
      await callLogin({ login: { userName: username, password } });
      refetchUserInfo();
      refetchUserRoles();
    },
    [callLogin, refetchUserInfo, refetchUserRoles]
  );

  useEffect(() => {
    setUser(userResponse);
    setUserRoles(userRolesResponse);
  }, [userRolesResponse, userResponse]);

  const logout = useCallback(() => {
    callLogout();
    setUser(undefined);
    setUserRoles(undefined);
  }, [callLogout, setUser, setUserRoles]);

  return (
    initialized && (
      <userContext.Provider
        value={{
          user,
          userRoles,
          login,
          loginError: getErrorState(loginError).message,
          logout,
          resetLoginError: resetLogin
        }}
      >
        {children}
      </userContext.Provider>
    )
  );
}
